@@ -24,7 +24,7 @@
  *
  * Author(s): Berin Lautenbach
  *
- * $Id: WinCAPICryptoKeyRSA.hpp 1125514 2011-05-20 19:08:33Z scantor $
+ * $Id: WinCAPICryptoKeyRSA.hpp 1350045 2012-06-13 22:33:10Z scantor $
  *
  */
 
@@ -175,6 +175,26 @@ public :
 
 	virtual const unsigned char * getOAEPparams(void) const;
 
+	/**
+	 * \brief Set the MGF
+	 *
+	 * By default, the library expects crypto implementations to perform
+	 * OAEP padding with MGF_SHA1.  This call allows the library (or user)
+	 * to set a different choice.
+	 *
+	 * @param mgf the MGF constant identifying the function to use
+	 */
+
+	virtual void setMGF(maskGenerationFunc mgf);
+
+	/**
+	 * \brief Get the MGF
+	 *
+	 * @returns the MGF constant in use
+	 */
+
+	virtual enum maskGenerationFunc getMGF(void) const;
+
 	/**
 	 * \brief Verify a SHA1 PKCS1 encoded signature
 	 *
