@@ -23,7 +23,7 @@
  * XSECEnv := Configuration class - used by the other classes to retrieve
  *            information on the environment they are working under
  *
- * $Id: XSECEnv.cpp 1125514 2011-05-20 19:08:33Z scantor $
+ * $Id: XSECEnv.cpp 1478615 2013-05-03 00:07:02Z scantor $
  *
  */
 
@@ -86,6 +86,18 @@ const XMLCh s_defaultXENCPrefix[] = {
 
 };
 
+const XMLCh s_defaultXENC11Prefix[] = {
+
+	chLatin_x,
+	chLatin_e,
+	chLatin_n,
+	chLatin_c,
+    chDigit_1,
+    chDigit_1,
+	chNull
+
+};
+
 const XMLCh s_defaultXKMSPrefix[] = {
 
 	chLatin_x,
@@ -132,6 +144,7 @@ XSECEnv::XSECEnv(DOMDocument *doc) {
 	mp_ecPrefixNS = XMLString::replicate(s_defaultECPrefix);
 	mp_xpfPrefixNS = XMLString::replicate(s_defaultXPFPrefix);
 	mp_xencPrefixNS = XMLString::replicate(s_defaultXENCPrefix);
+    mp_xenc11PrefixNS = XMLString::replicate(s_defaultXENC11Prefix);
 	mp_xkmsPrefixNS = XMLString::replicate(s_defaultXKMSPrefix);
 
 	m_prettyPrintFlag = true;
@@ -159,6 +172,7 @@ XSECEnv::XSECEnv(const XSECEnv & theOther) {
 	mp_ecPrefixNS = XMLString::replicate(theOther.mp_ecPrefixNS);
 	mp_xpfPrefixNS = XMLString::replicate(theOther.mp_xpfPrefixNS);
 	mp_xencPrefixNS = XMLString::replicate(theOther.mp_xencPrefixNS);
+    mp_xenc11PrefixNS = XMLString::replicate(s_defaultXENC11Prefix);
 	mp_xkmsPrefixNS = XMLString::replicate(theOther.mp_xkmsPrefixNS);
 
 	m_prettyPrintFlag = theOther.m_prettyPrintFlag;
@@ -207,6 +221,10 @@ XSECEnv::~XSECEnv() {
 		XSEC_RELEASE_XMLCH(mp_xencPrefixNS);
 	}
 
+	if (mp_xenc11PrefixNS != NULL) {
+		XSEC_RELEASE_XMLCH(mp_xenc11PrefixNS);
+	}
+
 	if (mp_xkmsPrefixNS != NULL) {
 		XSEC_RELEASE_XMLCH(mp_xkmsPrefixNS);
 	}
@@ -302,6 +320,15 @@ void XSECEnv::setXENCNSPrefix(const XMLCh * prefix) {
 
 }
 
+void XSECEnv::setXENC11NSPrefix(const XMLCh * prefix) {
+
+	if (mp_xenc11PrefixNS != NULL)
+		XSEC_RELEASE_XMLCH(mp_xenc11PrefixNS);
+
+	mp_xenc11PrefixNS = XMLString::replicate(prefix);
+
+}
+
 void XSECEnv::setXKMSNSPrefix(const XMLCh * prefix) {
 
 	if (mp_xkmsPrefixNS != NULL)
