@@ -26,7 +26,7 @@
  *
  * Author(s): Berin Lautenbach
  *
- * $Id: XSECW32Config.hpp 1140638 2011-06-28 14:29:08Z scantor $
+ * $Id: XSECW32Config.hpp 1350042 2012-06-13 22:29:17Z scantor $
  *
  */
 
@@ -183,7 +183,6 @@
 
 #	include <openssl/opensslv.h>
 #	if (OPENSSL_VERSION_NUMBER >= 0x00907000)
-
 #		define XSEC_OPENSSL_CONST_BUFFERS
 #		define XSEC_OPENSSL_HAVE_AES
 #       define XSEC_OPENSSL_HAVE_EC
@@ -195,6 +194,9 @@
 #       define XSEC_OPENSSL_HAVE_SHA2
 #       define XSEC_OPENSSL_HAVE_MGF1
 #	endif
+#	if (OPENSSL_VERSION_NUMBER >= 0x10001000)
+#		define XSEC_OPENSSL_HAVE_GCM
+#	endif
 
 #endif
 
