@@ -24,7 +24,7 @@
  *
  * Author(s): Berin Lautenbach
  *
- * $Id: siginf.cpp 1125514 2011-05-20 19:08:33Z scantor $
+ * $Id: siginf.cpp 1354660 2012-06-27 18:40:57Z scantor $
  *
  */
 
@@ -376,7 +376,7 @@ void outputSignatureInfo(DSIGSignature *sig, bool skipReferences) {
 
 void printUsage(void) {
 
-	cerr << "\nUsage: siging [options] <input file name>\n\n";
+	cerr << "\nUsage: siginf [options] <input file name>\n\n";
 	cerr << "     Where options are :\n\n";
 	cerr << "     --skiprefs/-s\n";
 	cerr << "         Skip information on references - output main sig info only\n\n";
