@@ -22,7 +22,7 @@
  *
  * TXFMXSL := Class that performs XPath transforms
  *
- * $Id: TXFMXSL.cpp 1125514 2011-05-20 19:08:33Z scantor $
+ * $Id: TXFMXSL.cpp 1493962 2013-06-17 22:32:41Z scantor $
  *
  */
 
@@ -187,8 +187,12 @@ void TXFMXSL::evaluateStyleSheet(const safeBuffer &sbStyleSheet) {
 
 	parser->setDoNamespaces(true);
 	parser->setCreateEntityReferenceNodes(true);
+	parser->setLoadExternalDTD(false);
 	parser->setDoSchema(true);
 
+	SecurityManager securityManager;
+	parser->setSecurityManager(&securityManager);
+
 	// Create an input source
 
 	MemBufInputSource* memIS = new MemBufInputSource ((const XMLByte*) txoh.buffer.rawBuffer(), txoh.offset, "XSECMem");
